/* $Id: vga.c,v 1.6 1998/11/23 20:37:12 marcus Exp $
******************************************************************************

   VGA ramdac driver

   Copyright (C) 1995-1997 Jason McMullan	[jmcc@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <kgi/maintainers.h>
#define	MAINTAINER	Jon_Taylor
#define	DRIVER_NAME	"VGA DAC driver"
#define	DRIVER_REV	"$Revision: 1.6 $"
#define	DEBUG_LEVEL	0

#include <kgi/module.h>
#include <kgi/kgi_commands.h>

#define	DAC_PRECISION	6

#include <IBM/vga_dac.h>
#include <IBM/clut.inc>

static struct kgi_ramdac ramdac = 
{
	"IBM",				/* Manufacturer		*/
	"VGA",				/* Model		*/
	GM_TEXT16 | GM_4BIT | GM_8BIT,	/* Supported modes	*/
	0,			 	/* Flags		*/
	{ 0xFFFF, 0xFFFF },		/* Maximum resolution	*/
	0,				/* priv pointer         */
	DAC_8BIT_PB,  			/* Device type		*/
	{       0,  32000000 },		/* Normal mode		*/
	{       0,         0 },		/* 2xclk mode		*/
	{
		{	  0,  32000000 },	/* KGIGT_TEXT16	*/
		{	  0,         0 },	/* KGIGT_TEXT32	*/
		{         0,  32000000 },	/* KGIGT_1BIT	*/
		{	  0,  32000000 },	/* KGIGT_4BIT	*/
		{         0,  32000000 },	/* KGIGT_8BIT	*/	
		{	  0,         0 },	/* KGIGT_15BIT	*/
		{ 	  0,         0 },	/* KGIGT_16BIT	*/
		{         0,         0 },	/* KGIGT_24BIT	*/
		{	  0,         0 }	/* KGIGT_32BIT	*/
	}
};

/* GGI interface */

int kgim_ramdac_init(struct kgi_card *card, struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " rev. " DRIVER_REV);
	return EOK;
}

void kgim_ramdac_done(struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " removed.");
}

int kgim_ramdac_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,
	enum kgi_tm_cmd *cmd)
{
	kgi_uint newclock = mode->dclk;
  
	switch (mode->request.graphtype) {

	case KGIGT_TEXT16:
		break;

	case KGIGT_1BIT:
		break;

	case KGIGT_4BIT:
		break;   

	case KGIGT_8BIT:
		break;
  
	default:
  		DEBUG2("couldn't handle graphtype %i", 
		       mode->request.graphtype);
		return -E(RAMDAC,MODE_CHECK);
	}

	switch (*cmd) {

	case CMD_PROPOSE:
		mode->lclk.mul = mode->clk.mul = 1;
		mode->lclk.div = mode->clk.div = 1;
		return EOK;

	case CMD_LOWER:

		if ((newclock < ramdac.clock[mode->request.graphtype].min) ||
			(newclock < ramdac.normal.min)) {

			DEBUG2("DCLK too low (%i Hz).", newclock);
			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if (ramdac.clock[mode->request.graphtype].max < newclock) {

			newclock = ramdac.clock[mode->request.graphtype].max;
		}
		mode->dclk = newclock;
		return EOK;

	case CMD_RAISE:
		if ((newclock > ramdac.clock[mode->request.graphtype].max) ||
			(newclock > ramdac.normal.max)) {

			DEBUG2("DCLK too high (%i Hz).", newclock);
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		if (newclock < ramdac.clock[mode->request.graphtype].min) {

			newclock = ramdac.clock[mode->request.graphtype].min;
		}

		if (newclock < ramdac.normal.min) {

			newclock = ramdac.normal.min;
		}

		mode->dclk = newclock;
		return EOK;

     case CMD_CHECK:
		if ((mode->lclk.mul != 1) || (mode->lclk.div != 1) ||
		    (mode->clk.mul != 1) || (mode->clk.div != 1)) {

			return -E(RAMDAC,MODE_CHECK);
		}

		if ((mode->dclk < ramdac.clock[mode->request.graphtype].min) ||
		    (mode->dclk < ramdac.normal.min)) {

			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if ((mode->dclk > ramdac.clock[mode->request.graphtype].max) ||
		    (mode->dclk > ramdac.normal.max)) {
			
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		return EOK;

	default:
		INTERNAL_ERROR;
		return -E(RAMDAC,BOGUS);
	}
}

void kgim_ramdac_set_mode(struct kgi_display *dpy, struct kgi_mode *mode)
{
	DAC_out8(dpy, 0xFF, DAC_PIXEL_MASK);
	dpy->set_clut = dac_set_clut;
}

int kgim_ramdac_command(struct kgi_graphic *graph, kgi_uint cmd, kgi_uint arg)
{
	/* This may set 
	 * -  Hardware cursor
	 * -  CLUT (or gamma correction table)
	 */
	kgi_clut clut;

	switch (cmd) { 

	case RAMDAC_SETCLUT:
		clut.size = 256;
		clut.data = (kgi_color *)(graph->io_buf);
		dac_set_clut(graph->dev.dpy, &clut, 0, 256);
		return 0;

	case RAMDAC_GETCLUT:
		clut.size = 256;
		clut.data = (kgi_color *)(graph->io_buf);
		graph->io_res = graph->io_buf;
		dac_get_clut(graph->dev.dpy, &clut, 0, 256);
		return 0;
	}
  	return -E(RAMDAC, NOSUP_ALWAYS_CANT);  /* The ramdac can't do this */
}
